/*
  * Copyright 2015 The CHOReVOLUTION project
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
package eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.embedded;

import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IProgressMonitor;

import eu.chorevolution.studio.eclipse.core.ChorevolutionCoreUtils;
import eu.chorevolution.studio.eclipse.core.internal.project.model.impl.ChorevolutionSynthesisProjectStructureFolder;
import eu.chorevolution.studio.eclipse.core.preferences.ChorevolutionServiceThingSourceModelPrefs;
import eu.chorevolution.studio.eclipse.core.preferences.ChorevolutionSynthesisSourceModelPrefs;
import eu.chorevolution.studio.eclipse.core.utils.syncope.InterfaceDescriptionType;
import eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.BindingComponentGenerator;
import eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.GeneratedArtifactSynthesisProcessor;
import eu.chorevolution.studio.eclipse.ui.handlers.wizard.model.ParticipantTableViewerRecord;

public class BindingComponentGeneratorEmbedded extends BindingComponentGenerator{
	private GeneratedArtifactSynthesisProcessor generatedArtifactSynthesisProcessor; 
	 
	private IProgressMonitor monitor;
	private List<ParticipantTableViewerRecord> providerParticipants;
	private boolean generateBindingComponentClient;
	private List<ParticipantTableViewerRecord> clientParticipants;
	
	public BindingComponentGeneratorEmbedded(List<ParticipantTableViewerRecord> providerParticipants,
			boolean generateBindingComponentClient, List<ParticipantTableViewerRecord> clientParticipants,
			GeneratedArtifactSynthesisProcessor generatedArtifactSynthesisProcessor, IProject project,
			IProgressMonitor monitor) {
		super(project);
		
		this.generatedArtifactSynthesisProcessor = generatedArtifactSynthesisProcessor;
		this.monitor = monitor;
		this.providerParticipants = providerParticipants;
		this.generateBindingComponentClient = generateBindingComponentClient;
		this.clientParticipants = clientParticipants;
	
	}

	@Override
	public void storeBindingComponents() throws Exception {
		monitor.beginTask("Binding Component Generator",  providerParticipants.size());

		for (ParticipantTableViewerRecord participantTableViewerRecord : providerParticipants) {
			if (monitor.isCanceled()) {
				return;
			}

			monitor.subTask("Generate Binding Component for Service: "
					+ participantTableViewerRecord.getRecordLabel());
			storeBindingComponentService(participantTableViewerRecord);
			monitor.worked(1);
		}


	}


	private void storeBindingComponentService(ParticipantTableViewerRecord participantTableViewerRecord) throws Exception  {
		// generate binding component only service that has interface description different to WSDL
		if (participantTableViewerRecord.getService() != null
                && participantTableViewerRecord.getService().getInterfaceDescriptionType() != null 
                && participantTableViewerRecord.getService().getInterfaceDescriptionType() == InterfaceDescriptionType.WSDL) {
            return;
        }
		IFolder participantFolder = ChorevolutionCoreUtils.createFolder(participantTableViewerRecord.getParticipant(),
				super.getProject().getFolder(ChorevolutionSynthesisProjectStructureFolder.SYNTHESIS_PROCESSOR_FOLDER_NAME)
						.getFolder(super.getPropertyValues()
								.get(ChorevolutionSynthesisSourceModelPrefs.PREF_SYNTHESISPROCESSOR_BINDINGCOMPONENT)
								.getValue()));
		IFile gidl = super.getProject().getFolder(super.getPropertyValues()
								.get(ChorevolutionSynthesisSourceModelPrefs.PREF_SERVICEINVENTORY_SERVICES)
								.getValue()).getFolder(participantTableViewerRecord.getService().getName()).getFolder(super.getPropertyValues()
										.get(ChorevolutionServiceThingSourceModelPrefs.PREF_SERVICE_THING_INTERFACE_DESCRIPTION)
										.getValue().toLowerCase()).getFile(participantTableViewerRecord.getService().getName() + "."
										+ participantTableViewerRecord.getService().getInterfaceDescriptionType().name()
										.toLowerCase());
		
/*
		VsbManager vsbManager = new VsbManager();
		VsbOutput response = vsbManager.generateWar(gidl.getRawLocation().makeAbsolute().toFile().getAbsolutePath(), ProtocolType.SOAP);
		
		if (response != null && response.war != null) {
			IFile bindingComonentFile =ChorevolutionCoreUtils.createFile(participantTableViewerRecord.getServiceProjectName() + "." + WAR_EXTENSION, participantFolder,
					response.war);
			ComponentData providerComponentData = generatedArtifactSynthesisProcessor.getProviderComponent(participantTableViewerRecord.getParticipant());
			
			generatedArtifactSynthesisProcessor.addBindingComponentToProviderComponent(
					providerComponentData,
					participantTableViewerRecord.getServiceProjectName(), 
					EMBEDDED_PREFIX_LCOATION+bindingComonentFile.getRawLocation().makeAbsolute().toOSString());
		}
		
		super.getProject().refreshLocal(IResource.DEPTH_ZERO, new NullProgressMonitor());
		*/
	}

}
