/*
  * Copyright 2015 The CHOReVOLUTION project
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
package eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.embedded;

import org.eclipse.core.resources.IProject;

import eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.ChoreographyValidator;
import eu.chorevolution.synthesisprocessor.bpmn2choreographyvalidator.Bpmn2ChoreographyValidator;
import eu.chorevolution.synthesisprocessor.bpmn2choreographyvalidator.Bpmn2ChoreographyValidatorRequest;
import eu.chorevolution.synthesisprocessor.bpmn2choreographyvalidator.Bpmn2ChoreographyValidatorResponse;

public class ChoreographyValidatorEmbedded extends ChoreographyValidator {
	private byte[] bpmn2Content;
	private byte[] typesContent;
	private Bpmn2ChoreographyValidatorResponse response;

	public ChoreographyValidatorEmbedded(IProject project, byte[] bpmn2Content, byte[] typesContent) {
		super(project);
		this.bpmn2Content = bpmn2Content;
		this.typesContent = typesContent;
	}

	@Override
	public boolean validateBpmn2ChoreographyDiagram() throws Exception {
		Bpmn2ChoreographyValidatorRequest bpmn2ChoreographyValidatorRequest = new Bpmn2ChoreographyValidatorRequest();
		bpmn2ChoreographyValidatorRequest.setBpmn2Content(bpmn2Content);
		bpmn2ChoreographyValidatorRequest.setBpmn2XSD(typesContent);
		this.response = new Bpmn2ChoreographyValidator().validate(bpmn2ChoreographyValidatorRequest);
		return response.isValid();
	}

	@Override
	public String getErrors() {
		if (this.response != null) {
			StringBuilder stringBuilder = new StringBuilder();
			stringBuilder.append("\n");
			for(String error : response.getErrors()){
				stringBuilder.append(error+"\n");
			}
			return stringBuilder.toString();
		}
		return "";
	}

}
